"""
This module contains the functions used to parse different MS/MS search
results into simple dictionaries
"""

from cElementTree import iterparse
from pprint import pprint
import sys

def parse_xtandem(tandem_output):
    """Parses the spectra data from X!Tandem output to a dictionary with
    spectrum ids as keys and a dictionary with attribute vaule as values
    """

    spectra = {}

    for event, group in iterparse(tandem_output, events=("start", "end")):
        if event == "end" and \
                group.tag == "group" and \
                group.get("type") == "model":        
            spectrum_id = group.get("id")
            # TODO: link multiple peptide to multiple protein
            for protein in group.findall("./protein"):
                spectra[spectrum_id] = {"protein":
                    protein.get("label")}
            # TODO: move this to unit testing
                if len(protein.findall("peptide")) != 1:
                    raise NameError,"\
                            Check your code, there is more than\
                            one peptide per protein in\
                            X!Tandem output format\
                            "
                for peptide in protein.findall("peptide/domain"):
                    spectra[spectrum_id] = {"peptide": peptide.get("seq")}

    return spectra

def parse_mascot(mascot_output):
    """Parses the spectra data from Mascot output to a dictionary with
    spectrum ids as keys and a dictionary with attribute value as values"""
    msprotein = {} 
    mascot_tag = "{http://www.matrixscience.com/xmlns/schema/mascot_search_results_2}"
    for event, elem in iterparse(mascot_output,("start", "end")):
        if event == "start" and elem.tag == mascot_tag+"protein":
             gi = elem.get("accession")
        if event == "start" and elem.tag == mascot_tag+"peptide":
             peptide_query = elem.get("query")
             peptide_rank = elem.get("rank")
             isbold = elem.get("isbold")			
             msprotein[peptide_query,peptide_rank,\
                                 isbold,gi] = {"protein":gi}
        if event == "start" and elem.tag == mascot_tag+"pep_seq":
            pep_seq = elem.text
            msprotein[peptide_query,peptide_rank,\
                                isbold,gi] = {"protein":gi,\
                                                "peptide":pep_seq}
        if event == "start" and elem.tag == mascot_tag+"unassigned":
            break
    return msprotein

def get_values(mascot_parsed_values):    
    spectra = {}
    for key in mascot_parsed_values:
        spectrum_number = key[0].strip()
        try:
            spectra[spectrum_number].append(mascot_parsed_values[key])
        except KeyError:
            spectra[spectrum_number] = []
            spectra[spectrum_number].append(mascot_parsed_values[key])
    return spectra


def parse_omssa(omssa_output):
    mspeptide = {}
    msprotein = {}
    spectra = {}
    omssa_tag = "{http://www.ncbi.nlm.nih.gov}"
    for event, group in iterparse(omssa_output,\
                                    events = ("start","end")):
        if event == "start" and \
            group.tag == omssa_tag+"MSHitSet_number":
            spectrum_id = group.text
        for protein in group.findall(omssa_tag+"MSPepHit_gi"):
            gi = protein.text
            mspeptide[spectrum_id,gi] = {"protein":gi}
        if event == "start" and \
            group.tag == omssa_tag+"MSHits_pepstring":
            pep_seq = group.text
            msprotein[spectrum_id,pep_seq,gi] = {"protein":gi,"peptide":pep_seq} 
    for prot in msprotein:
        spectrum = prot[0].strip()
        try:
            spectra[spectrum].append(msprotein[prot])
        except:
            spectra[spectrum] = []
            spectra[spectrum].append(msprotein[prot])
    return spectra


#if __name__ == '__main__':
#    try:
#        pprint(parse_tandem_spectra(sys.argv[1]))
#    except IndexError:
#        print "You have to pass X!Tandem output as an argument"
#
#if __name__ == "__main__":
#     mascot_parsed_values = parse_mascot_spectra(sys.argv[1])
#     spectra = get_values(mascot_parsed_values)
#     pprint(spectra)
#
#if __name__ == '__main__':
#
#    omssaParsedResults= parse_omssa_spectra(sys.argv[1])
#    pprint(omssaParsedResults)







