#!/usr/bin/env python

from cElementTree import iterparse
from pprint import pprint
import sys

def parse_mascot_spectra(mascot_output):
    """Parses the spectra data from Mascot output to a dictionary with
    spectrum ids as keys and a dictionary with attribute value as values"""
    msprotein = {} 
    mascot_tag = "{http://www.matrixscience.com/xmlns/schema/mascot_search_results_2}"
    for event, elem in iterparse(mascot_output,("start", "end")):
        if event == "start" and elem.tag == mascot_tag+"protein":
             gi = elem.get("accession")
        if event == "start" and elem.tag == mascot_tag+"peptide":
             peptide_query = elem.get("query")
             peptide_rank = elem.get("rank")
             isbold = elem.get("isbold")			
             msprotein[peptide_query,peptide_rank,\
                                 isbold,gi] = {"protein":gi}
        if event == "start" and elem.tag == mascot_tag+"pep_seq":
            pep_seq = elem.text
            msprotein[peptide_query,peptide_rank,\
                                isbold,gi] = {"protein":gi,\
                                                "peptide":pep_seq}
        if event == "start" and elem.tag == mascot_tag+"unassigned":
            break
    return msprotein
def get_values(mascot_parsed_values):    
    spectra = {}
    for key in mascot_parsed_values:
        spectrum_number = key[0].strip()
        try:
            spectra[spectrum_number].append(mascot_parsed_values[key])
        except KeyError:
            spectra[spectrum_number] = []
            spectra[spectrum_number].append(mascot_parsed_values[key])
    return spectra
if __name__ == "__main__":
     mascot_parsed_values = parse_mascot_spectra(sys.argv[1])
     spectra = get_values(mascot_parsed_values)
     pprint(spectra)
