#!/usr/bin/env python
# encoding: utf-8
"""
parsers.py

Created by Danny Navarro on 2008-04-04.
Copyright (c) 2008 __IOB__. All rights reserved.

This module contains the parsing functions for different
search engines that will be created while parsing
MS/MS search results
"""

import sys
#XXX: import elementtree if cElementTree is not there
from cElementTree import iterparse
from pprint import pprint

from pdisco.msparsers.targets import \
Spectrum, Peptide, PSM


def parse_xtandem_results(xtandem_output):
    """This generator returns an iterator of PSMs from X! Tandem
    results
    """

    for event, group in iterparse(
            xtandem_output,
            events=("start", "end")
            ):

        if event == "end" and \
            group.tag == "group" and \
            group.get("type") == "model":

            spectrum_id = (group.get("id"))

            parent_peak = (
                    float(group.get("mh")), # mass 
                    float(group.get("sumI")), # log of parent peaks
                    int(group.get("z")) # charge
                    )

# TODO: Find out how to connect the peak_list to spectrum. group
# type is "support"

#            for axis in group.findall("./GAML:trace/GAML:Xdata"):
#                # Only one X axis
#                assert len(axis) == 1
#                mzs = tuple(axis[0].text)
#
#            for axis in group.findall("./GAML:trace/GAML:Ydata"):
#                assert len(axis) == 1
#                intensities = tuple(axis[0].text)
#
#            peak_list = tuple(zip(mzs, intensities))

            for protein in group.findall("protein"):
                protein_label = protein.get("label")

                # There should be one peptide tag per protein
                assert len(protein.findall("peptide")) == 1

                for pep in protein.findall("peptide/domain"):
                    # mass plus proton
                    pep_sequence = pep.get("seq")
                    pep_mass = pep.get("mh")
                    missed_cleavages = \
                        int(pep.get("missed_cleavages"))

                    psm_dmass = pep.get("delta")
                    psm_score = pep.get("hyperscore")
                    psm_expect = pep.get("expect")

                    peptide = Peptide(
                            pep_sequence,
                            pep_mass,
                            missed_cleavages,
                            protein_label
                            )

                    spectrum = Spectrum(
                            spectrum_id,
                            parent_peak
                            )

                    #spectrum.peak_list = peak_list

                    psm = PSM(
                            peptide,
                            spectrum,
                            psm_dmass,
                            psm_score,
                            psm_expect
                            )

                    yield psm

def main():
    for psm in parse_xtandem_results(sys.argv[1]):
        print psm.score


if __name__ == '__main__':
    main()
